/********************************************************************************************************************
All ADuC706x library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

Status:			Tested in simulation.
Tools:			IAR compiler.
Hardware:		ADuC7060/61/62.
Description:		This file contains the main routines for reading the external RTD via ADC0
 			It contains the User interface functions for handling user requests.
                        The linearization functions are called from here. The linearization functions are called from:
                        RTDDirect.c or, RTDpwl.c or RTDSing.c . Only one of these 3 files should be included in any project build.
			When using the ADuC7060 Evaluation board, it will digitize the RTD reading.
			To measure the RTD, the following switch settings must be in place:
				 S4-1  			-	ON	
				 S4-2  			-	ON
				 S4-3  			-	ON
				 S4-4  			- 	ON
				 S4-5 to S4-8 	        -	OFF
Responsibility:
	Group:		IAC Apps.
	Person:		Mike Looney.
Modifications:
	2008/05/30	Design Start.
************************************************************************************************************************/




#include <Analogdevices\ioaduc7060.h >
#include <math.h>
#include <intrinsics.h>
#include <stdio.h>
#include <string.h>
#include <C:\ADuC7XXXV0.1\ADuC706X\code\IAR\RTD_PieceWise_Linearization\RTDdefs.h>

float Trtd = 0.0;   				// temperature of RTD
float Rrtd = 0.0;				// Resistance of RTD
float fADC0norm = 0.0;

//float fscale0 = 351.25;                       // Default value when gain = 16
float fscale0 = 175.625;                        // Default value when gain = 32
float foffset0 = 0.0;
float fRprevcal;

// Application specifc Flags
unsigned char newADCdata;                       // 1 means new ADC data available
unsigned char bSerialIn = 0;                    // Set to 1 if data received by the UART
unsigned char ucFahrenheit = 0;                 // Select fahrenheit or celcius scale
unsigned char ucMenuFlag = 0;                   // Set by toggling XIRQ0 - will call main menu screen again


// resistance at previous cal point

volatile unsigned char bSendResultToUART = 0;	// Flag used to indicate ADC0 resutl ready to send to UART	
unsigned char szTemp[64] = "";			// Used to store ADC0 result before printing to UART
unsigned char ucTxBufferEmpty  = 0;		// Used to indicate that the UART Tx buffer is empty
volatile unsigned long ulADC0Result = 0;	// Variable that ADC0DAT is read into in ADC0 IRQ
volatile unsigned char ucUserEntry = 0;         // Character received from User interface

void delay(int);
void Temperature (void);
void SerialCommand(void);
void PrintString (void);

extern signed char Cal (float Rcal);
extern float T_rtd (float r);
extern float R_rtd (float r);
extern float Tmin_rtd (void);
extern float Tmax_rtd (void);
extern float Rmin_rtd (void);
extern float Rmax_rtd (void);
extern void Calibrate (void);


// Application specifc Flags
unsigned char newADCdata;                       // 1 means new ADC data available


// Main Routine - Initializes device and calls user interface
int main(void)
{
    POWKEY1 = 0x1;
    POWCON0 = 0x78;		                // Set core to max CPU speed of 10.24Mhz
    POWKEY2 = 0xF4;

    fRprevcal = 0;                               // previous cal resistance (no prev.cal)

    // turn on excitation current source
    IEXCON = BIT1 + BIT6; 	                // Enable Excitation Current Source 0 - 200uA
    GP0DAT = BIT24;
    GP1DAT  = BIT30 + BIT29;		                // Configure P1.6 as an output to drive the LED
    __enable_interrupt();
    // Initialize the UART for 9600-8-N
    GP1CON = BIT0 + BIT4;                       // Select UART functionality for P1.0/P1.1
    COMCON0 = BIT7;			        // Enable access to COMDIV registers
    COMDIV0 = 0x21;			        // Set baud rate to 9600.
    COMDIV1 = 0x00;
    //COMDIV2 = 0x21 + BIT11;                   // Enable fractional divider for more accurate baud rate setting
    COMCON0 = BIT0 + BIT1 + BIT2;
    COMIEN0 = BIT0 + BIT1;	                // Enable UART interrupts when Rx full and Tx buffer empty.
    IRQEN = BIT11;                              // Enable  UART interrupts


    // Configure ADC0 for continuous conversions, 100hz, AIN0/1 Differential inputs
    ADCMSKI = BIT0;				// Enable ADC0 result ready interrupt source
    ADCFLT  = 0x4F; 				// 100Hz no filtering, Chop off, No averaging
    ADC0CON = BIT10 + BIT15;	                // Gain = 32, Unipolar, enable ADC0, Int Reference
    ADCCFG = 0;

    // Offest Self Calibration
    ADCMDE  = BIT2 + BIT4 + BIT7;	
    while((ADCSTA & BIT0) != BIT0){}	        // Wait for Calibration routine to complete

    ADC0CON = BIT0 + BIT2 + BIT4                // Gain = 32, Unipolar, enable ADC0, Int Reference (clear BIT0 for Gain = 16)
            + BIT10 + BIT15;
    ADCMDE  = BIT0 + BIT7;			// Enable Continuous conversion mode
    IRQEN |= BIT10 + BIT11;                     // Enable ADC interrupts
    IRQCONE = BIT1;			        // Rising edge of XIRQ0
    IRQEN |=  BIT13;			        // Enable External IRQ0

    // Output main menu
    SerialCommand();                            //
    bSendResultToUART = 0;
    while (1)
    {
      if (newADCdata == 1)
        {
	    Temperature();
            bSendResultToUART = 1;
	}
      delay (0x8FFFF);
      if (bSendResultToUART == 1)               // Is there an ADC0 result ready for UART transmission?
	{	
	    if (ucFahrenheit == 1)
            {
              sprintf ( (char*)szTemp, "Temperature : %+8.2fF,\r\n",(1.8*Trtd+32));// Send the Temperature Result to the UART
              PrintString();
            }
            else
            {
                sprintf ( (char*)szTemp, "Temperature : %+8.2fC,\r\n",Trtd );// Send the Temperature Result to the UART
                PrintString();
            }
	    sprintf ( (char*)szTemp, "ADC0 Result (hex) : %8lX,\r\n",ulADC0Result );// Send the ADC0 Result to the UART
            PrintString();
	    sprintf ( (char*)szTemp, "RTD value : %+8.3fohms,\r\n\n\n",Rrtd );// Send the RTD resistence value to the UART
            PrintString();
	    IRQEN |= BIT10;
   	  }
      if (ucMenuFlag == 1)                         // Call up the menu screen again
        {
           SerialCommand();
           ucMenuFlag = 0;
        }
    }
}


void delay (int length)
{
    while (length >0)
    length--;
}
void Temperature ()
{
   newADCdata = 0;
  // First calculate the resistance across the RTD

  Rrtd = fADC0norm*fscale0 + foffset0;      // RTD resistance [ohms]
  // Next the temperature value is calculated using Rtd resistance
  /************* Simple Linear equation:************************
  // At 35 degree Celsius, the equivalent resistance of a PT100 is 113.607 Ohm.
  GP1DAT ^= BIT21;
  Trtd = (Rrtd - 100.0) * (35.0/13.607);
  GP1DAT ^= BIT21;
  **************************************************************/
  Trtd = T_rtd(Rrtd);               // RTD temperature [degC]
}

void SerialCommand ()
{

  // display main menu
  sprintf ( (char*)szTemp, "ADuC7060 RTD Interface Program\r\n");// Send the Temperature Result to the UART
  PrintString();
  sprintf ( (char*)szTemp, "MAIN MENU....\r\n");// Send the Temperature Result to the UART
  PrintString();
  sprintf ( (char*)szTemp, "  C: display units in Celcius\r\n");// Send the Temperature Result to the UART
  PrintString();
  sprintf ( (char*)szTemp, "  F: display units in Fahrenheit\r\n");// Send the Temperature Result to the UART
  PrintString();
  sprintf ( (char*)szTemp, " X: calibrate\r\n");// Send the Temperature Result to the UART
  PrintString();
  sprintf ( (char*)szTemp, " any other key: resume normal operation\r\n");// Send the Temperature Result to the UART
  PrintString();
  sprintf ( (char*)szTemp, " select option:");// Send the Temperature Result to the UART
  PrintString();
  while (bSerialIn == 0)
  {}
    // restore serial_in to zero, ready for next serial interrupt
  bSerialIn = 0;
  // get keystroke & perform corresponding action
  switch (ucUserEntry)
  {
    case 'c' :
    case 'C' : COMTX = 0x63;
               while (ucTxBufferEmpty == 0)
                {
                }
               ucFahrenheit = 0;
               break;
    case 'f' :
    case 'F' : COMTX = 0x66;
               while (ucTxBufferEmpty == 0)
                {
                }
               ucFahrenheit = 1;
               break;
    case 'x' :
    case 'X' : COMTX = 0x78;
               while (ucTxBufferEmpty == 0)
                {
                }
               Calibrate();
               break;
    default  : break;
  }
  ADCMDE  = BIT0 + BIT7;			// Enable Continuous conversion mode
}
void PrintString (void)
{
      unsigned char i = 0;
      unsigned char nLen = 0;

      nLen = strlen((char*)szTemp);
      for ( i = 0 ; i < nLen ; i++ )	    // loop to send ADC0 result
      {
         COMTX = szTemp[i];
         ucTxBufferEmpty = 0;
         while (ucTxBufferEmpty == 0)
         {
         }
      }
}


__irq __arm void IRQ_Handler(void)		
{
    unsigned long IRQSTATUS = 0;
    unsigned char ucCOMIID0 = 0;
   // float fADC0temp = 0.0;

    IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
    if ((IRQSTATUS & BIT11) == BIT11)	//UART interrupt source
      {
          ucCOMIID0 = COMIID0;
	  if ((ucCOMIID0 & 0x2) == 0x2)	  // Transmit buffer empty
	    {
	        ucTxBufferEmpty = 1;
	    }
          if ((ucCOMIID0 & 0x4) == 0x4)	  // Receive buffer empty
	    {
	        bSerialIn = 1;
                ucUserEntry = COMRX;
	    }
      }
    if ((IRQSTATUS & BIT10) == BIT10)	//If ADC0 interrupt source
      {
	// if ADCnorm variables are ready for new data..
  	  if (newADCdata==0)
	    {
		ulADC0Result = ADC0DAT;	// Read ADC0 conversion result
                fADC0norm = (float)(ulADC0Result/16777216.0);
  	    }
          newADCdata = 1;		
	  GP1DAT ^= BIT22;		
	  newADCdata = 1;
	  IRQCLR = BIT10;
      }
    if ((IRQSTATUS & BIT13) == BIT13)	//External Interrupt0 source
      {
	   IRQCLRE = BIT13;			   // Clear External Interrupt 0
           ADCMDE  = BIT1 + BIT7;		   // Enable Single conversion mode
           ucMenuFlag = 1;                         // Call up the menu screen again
      }
}

